//
//  OrderListAllView.swift
//  SodaLive
//
//  Created by klaus on 2023/08/24.
//

import SwiftUI

struct OrderListAllView: View {
    
    @StateObject var viewModel = OrderListAllViewModel()
    
    var body: some View {
        BaseView(isLoading: $viewModel.isLoading) {
            VStack(spacing: 0) {
                HStack(spacing: 0) {
                    Button {
                        AppState.shared.back()
                    } label: {
                        Image("ic_back")
                            .resizable()
                            .frame(width: 20, height: 20)
                        
                        Text("구매목록")
                            .font(.custom(Font.bold.rawValue, size: 18.3))
                            .foregroundColor(Color(hex: "eeeeee"))
                    }
                    
                    Spacer()
                }
                .padding(.horizontal, 13.3)
                .frame(height: 50)
                .background(Color.black)
                
                ScrollViewReader { reader in
                    ScrollView(.vertical, showsIndicators: false) {
                        LazyVStack(spacing: 10.7) {
                            ScrollerToTop(reader: reader, scrollOnChange: $viewModel.scrollToTop)
                            
                            ForEach(0..<viewModel.orderList.count, id: \.self) { index in
                                let item = viewModel.orderList[index]
                                OrderListItemView(item: item)
                                    .contentShape(Rectangle())
                                    .onTapGesture {
                                        AppState
                                            .shared
                                            .setAppStep(step: .contentDetail(contentId: item.contentId))
                                    }
                                    .padding(.horizontal, 13.3)
                                    .onAppear {
                                        if index == viewModel.orderList.count - 1 {
                                            viewModel.getOrderList()
                                        }
                                    }
                            }
                        }
                    }
                }
                .padding(.top, 13.3)
            }
            .onAppear {
                viewModel.getOrderList()
            }
            .popup(isPresented: $viewModel.isShowPopup, type: .toast, position: .bottom, autohideIn: 2) {
                HStack {
                    Spacer()
                    Text(viewModel.errorMessage)
                        .padding(.vertical, 13.3)
                        .frame(width: screenSize().width - 66.7, alignment: .center)
                        .font(.custom(Font.medium.rawValue, size: 12))
                        .background(Color(hex: "9970ff"))
                        .foregroundColor(Color.white)
                        .multilineTextAlignment(.leading)
                        .cornerRadius(20)
                        .padding(.bottom, 66.7)
                    Spacer()
                }
            }
        }
    }
}
